import sys
import os
os.environ["adaptive_binning_chisquared_2sam"]=os.getcwd()
#sys.path.insert(0, './GoF')
import chi2_regular_binning, chi2_adaptive_binning, chi2_adaptive_binning_systematics , chi2_plots, calling_chi2_example
